Attribute VB_Name = "modFontInfo"
Option Explicit

''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Copyright 1996-2004 VBnet, Randy Birch, All Rights Reserved.
' Some pages may also contain other copyrights by the author.
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
' Distribution: You can freely use this code in your own
'               applications, but you may not reproduce
'               or publish this code on any web site,
'               online service, or distribute as source
'               on any media without express permission.
''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

Public Const LF_FACESIZE = 32
Public Const SPI_GETNONCLIENTMETRICS = 41
Public Const LOGPIXELSY As Long = 90

Public Type LOGFONT
    lfHeight As Long
    lfWidth As Long
    lfEscapement As Long
    lfOrientation As Long
    lfWeight As Long
    lfItalic As Byte
    lfUnderline As Byte
    lfStrikeOut As Byte
    lfCharSet As Byte
    lfOutPrecision As Byte
    lfClipPrecision As Byte
    lfQuality As Byte
    lfPitchAndFamily As Byte
    lfFaceName(0 To LF_FACESIZE - 1) As Byte
End Type

Public Type NONCLIENTMETRICS
    cbSize As Long
    iBorderWidth As Long
    iScrollWidth As Long
    iScrollHeight As Long
    iCaptionWidth As Long
    iCaptionHeight As Long
    lfCaptionFont As LOGFONT
    iSMCaptionWidth As Long
    iSMCaptionHeight As Long
    lfSMCaptionFont As LOGFONT
    iMenuWidth As Long
    iMenuHeight As Long
    lfMenuFont As LOGFONT
    lfStatusFont As LOGFONT
    lfMessageFont As LOGFONT
End Type

Public Enum NonClientFonts
   SystemCaptionFont = 1
   SystemCaptionSmallFont = 2
   SystemMenuFont = 3
   SystemMessageFont = 4
   SystemStatusFont = 5
End Enum

'type to return info from the
'wrapper routines
Public Type SYSTEMFONTINFO
   name As String
   size As Single
   bold As Boolean
   italic As Boolean
   underline As Boolean
   strikeout As Boolean
End Type

Private Declare Function GetDeviceCaps Lib "gdi32" _
  (ByVal hdc As Long, _
   ByVal nIndex As Long) As Long

Private Declare Function SystemParametersInfo Lib "user32" _
   Alias "SystemParametersInfoA" _
  (ByVal uAction As Long, _
   ByVal uParam As Long, _
   lpvParam As Any, _
   ByVal fuWinIni As Long) As Long
   
Private Declare Function GetDesktopWindow Lib "user32" () As Long
Private Declare Function GetWindowDC Lib "user32" (ByVal hwnd As Long) As Long
   
   
Public Function GetSystemFontInfo(nFont As NonClientFonts) As SYSTEMFONTINFO

   Dim ncm As NONCLIENTMETRICS
   Dim sfi As SYSTEMFONTINFO
   Dim tmp As String
   Dim hdc As Long
   
   hdc = GetWindowDC(GetDesktopWindow())
   
  'set the size of the structure
   ncm.cbSize = Len(ncm)
   
  'get the non-client metrics for the system
   If SystemParametersInfo(SPI_GETNONCLIENTMETRICS, _
                           ncm.cbSize, _
                           ncm, _
                           0) = 1 Then
                           
                           
     'For the font of interest, extract data
     'in a similar manner from the returned
     'NONCLIENTMETRICS structure:
     '1 - font name is returned as Unicode,
     '    so convert, stripping trailing null
     '
     '2 - the size needs to be calculated from
     '    the lfHeight returned. For example, if
     '    lfHeight = -11, the font size is 8 points
     '
     '3 - if the weight is > 400, the font is
     '    considered medium-to-bold
     '
     '4 - italic, underline and strikeout are
     '    true when 1 is returned
     Select Case nFont
         Case SystemCaptionFont
         
            tmp = StrConv(ncm.lfCaptionFont.lfFaceName, vbUnicode)
            With sfi
               .name = Left$(tmp, InStr(tmp, Chr$(0)) - 1)
               .size = -MulMul(ncm.lfCaptionFont.lfHeight, GetDeviceCaps(hdc, LOGPIXELSY), 72)
               .bold = ncm.lfCaptionFont.lfWeight > 400
               .italic = ncm.lfCaptionFont.lfItalic = 1
               .underline = ncm.lfCaptionFont.lfUnderline = 1
               .strikeout = ncm.lfCaptionFont.lfStrikeOut = 1
            End With
      
         Case SystemCaptionSmallFont
         
            tmp = StrConv(ncm.lfSMCaptionFont.lfFaceName, vbUnicode)
            With sfi
               .name = Left$(tmp, InStr(tmp, Chr$(0)) - 1)
               .size = -MulMul(ncm.lfSMCaptionFont.lfHeight, GetDeviceCaps(hdc, LOGPIXELSY), 72)
               .bold = ncm.lfSMCaptionFont.lfWeight > 400
               .italic = ncm.lfSMCaptionFont.lfItalic = 1
               .underline = ncm.lfSMCaptionFont.lfUnderline = 1
               .strikeout = ncm.lfSMCaptionFont.lfStrikeOut = 1
            End With
      
         Case SystemMenuFont
         
            tmp = StrConv(ncm.lfMenuFont.lfFaceName, vbUnicode)
            With sfi
               .name = Left$(tmp, InStr(tmp, Chr$(0)) - 1)
               .size = -MulMul(ncm.lfMenuFont.lfHeight, GetDeviceCaps(hdc, LOGPIXELSY), 72)
               .bold = ncm.lfMenuFont.lfWeight > 400
               .italic = ncm.lfMenuFont.lfItalic = 1
               .underline = ncm.lfMenuFont.lfUnderline = 1
               .strikeout = ncm.lfMenuFont.lfStrikeOut = 1
            End With
      
      
         Case SystemMessageFont
         
            tmp = StrConv(ncm.lfMessageFont.lfFaceName, vbUnicode)
            With sfi
               .name = Left$(tmp, InStr(tmp, Chr$(0)) - 1)
               .size = -MulMul(ncm.lfMessageFont.lfHeight, GetDeviceCaps(hdc, LOGPIXELSY), 72)
               .bold = ncm.lfMessageFont.lfWeight > 400
               .italic = ncm.lfMessageFont.lfItalic = 1
               .underline = ncm.lfMessageFont.lfUnderline = 1
               .strikeout = ncm.lfMessageFont.lfStrikeOut = 1
            End With
               
         Case SystemStatusFont
         
            tmp = StrConv(ncm.lfStatusFont.lfFaceName, vbUnicode)
            With sfi
               .name = Left$(tmp, InStr(tmp, Chr$(0)) - 1)
               .size = -MulMul(ncm.lfStatusFont.lfHeight, GetDeviceCaps(hdc, LOGPIXELSY), 72)
               .bold = ncm.lfStatusFont.lfWeight > 400
               .italic = ncm.lfStatusFont.lfItalic = 1
               .underline = ncm.lfStatusFont.lfUnderline = 1
               .strikeout = ncm.lfStatusFont.lfStrikeOut = 1
            End With
      
         Case Else
      End Select

      GetSystemFontInfo = sfi
      
   End If

End Function


Public Function MulMul(arg1 As Long, arg2 As Long, arg3 As Long) As Integer
 
  'A weird name for a function;
  'actually, it is based on the reverse of MulDiv
  '(the multiple divide C macro) and since it
  'returns the opposite data, I named it MulMul
  '(though there is no multiplication in it!).
  'I have no idea what its real corresponding
  'name would be in C.
   Dim tmp As Single
   
   tmp = arg2 / arg3
   tmp = arg1 / tmp
   
   MulMul = CInt(tmp)
   
End Function

Public Function MakeSystemFont(Obj As Object, Optional nFont As NonClientFonts = SystemMessageFont)
    Dim sfi As SYSTEMFONTINFO

    On Error Resume Next

    sfi = GetSystemFontInfo(nFont)
    With Obj.Font
        .name = sfi.name
        .size = sfi.size
        .bold = sfi.bold
        .italic = sfi.italic
        .underline = sfi.underline
        .Strikethrough = sfi.strikeout
        .Charset = &H405
    End With
End Function
