VERSION 5.00
Begin VB.UserControl ODListBox 
   BorderStyle     =   1  'Fixed Single
   ClientHeight    =   1785
   ClientLeft      =   0
   ClientTop       =   0
   ClientWidth     =   2595
   ScaleHeight     =   119
   ScaleMode       =   3  'Pixel
   ScaleWidth      =   173
   ToolboxBitmap   =   "ODListBox.ctx":0000
   Begin VB.PictureBox picCanvas 
      AutoRedraw      =   -1  'True
      BorderStyle     =   0  'None
      Height          =   372
      Left            =   0
      ScaleHeight     =   375
      ScaleWidth      =   2415
      TabIndex        =   0
      Top             =   0
      Visible         =   0   'False
      Width           =   2412
   End
End
Attribute VB_Name = "ODListBox"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Attribute VB_Description = "axMultiLine ListBox"
'----------------------------------------------------------------
'
' OwnerDraw ListBox & ComboBox Controls
'
' ODListBox (ODListBox.ctl)
'
'----------------------------------------------------------------
'
' Author: Eduardo A. Morcillo
' E-Mail: emorcillo@mvps.org
' Web Page: http://www.mvps.org/emorcillo
'
' Distribution: You can freely use this code in your own
'               applications but you can't publish this code in a
'               web site, online service, or any other media,
'               without my express permission.
'
' Usage: at your own risk.
'
' Tested on:
'            * Windows XP Pro SP1
'            * VB6 SP5
'
' History:
'          01/02/2004 - This code was released
'
'---------------------------------------------------------------------------------------
Option Explicit

Implements ICallback

' ----==== Properties ====----

Private m_oDraw As IDrawListItem
Private m_lSelFore As Long
Private m_lSelBack As Long
Private m_bSorted As Boolean
Private m_lBorderStyle As Long

' ----==== Private fields ====----

Private m_lListBox As Long       ' Handle of ListBox window
Private m_lWndProcLB As Long     ' Window procedure of ListBox
Private m_lWndProc As Long       ' Window procedure of UserControl
Private m_oItems As Collection   ' Items collection
Private m_oCanvas As Canvas      ' Canvas object used to draw the items

' ----==== Enums ====----

Enum BorderStyle
   bsNone
   bsSingle
   bsFlat
   bs3D
End Enum

' ----==== Events ====----

Event Click()
Attribute Click.VB_UserMemId = -600
Event DblClick()
Attribute DblClick.VB_UserMemId = -601
Event KeyDown(KeyCode As Integer, Shift As Integer)
Attribute KeyDown.VB_UserMemId = -602
Event KeyPress(KeyAscii As Integer)
Attribute KeyPress.VB_UserMemId = -603
Event KeyUp(KeyCode As Integer, Shift As Integer)
Attribute KeyUp.VB_UserMemId = -604
Event MouseDown(Button As Integer, Shift As Integer, X As Single, Y As Single)
Event MouseMove(Button As Integer, Shift As Integer, X As Single, Y As Single)
Event MouseUp(Button As Integer, Shift As Integer, X As Single, Y As Single)
Event Scroll()

'----------------------------------------------------------------
' Procedure : Add
' Purpose   : Adds a new item to the ListBox
'----------------------------------------------------------------
'
Public Function Add( _
   ByVal Text As String, _
   Optional ByVal Image As StdPicture, _
   Optional ByVal SelectedImage As StdPicture, _
   Optional ByVal Font As StdFont, _
   Optional ByVal Index, _
   Optional ByVal Key) As ListItem
Dim lIndex As Long
Dim oItem As ListItem

   If Font Is Nothing Then Set Font = UserControl.Font
   
   ' Create a new ListItem object
   Set oItem = New ListItem
   oItem.Initialize Text, Image, SelectedImage, UserControl.ForeColor, UserControl.BackColor, m_lSelFore, m_lSelBack, Key, Me
   
   ' Add the item to the ListBox
   If IsMissing(Index) Or IsEmpty(Index) Then
      lIndex = SendMessage(m_lListBox, LB_ADDSTRING, 0, ByVal ObjPtr(oItem))
   Else
      lIndex = SendMessage(m_lListBox, LB_INSERTSTRING, Index - 1, ByVal ObjPtr(oItem))
   End If

   If lIndex < 0 Then
      ' There was an error adding the item
      Err.Raise 381
   Else
      
      ' Add the item to the collection
      If lIndex >= m_oItems.Count Then
         
         If IsMissing(Key) Then
            m_oItems.Add oItem
         Else
            m_oItems.Add oItem, Key
         End If
         
      Else
         If IsMissing(Key) Then
            m_oItems.Add oItem, , lIndex + 1
         Else
            m_oItems.Add oItem, Key, lIndex + 1
         End If
      End If
        
   End If

   Set Add = oItem
   
End Function

'----------------------------------------------------------------
' Procedure : BackColor
' Purpose   : Returns the default background color
'----------------------------------------------------------------
'
Public Property Get BackColor() As OLE_COLOR
Attribute BackColor.VB_UserMemId = -501
   BackColor = UserControl.BackColor
End Property

'----------------------------------------------------------------
' Procedure : BackColor
' Purpose   : Sets the default background color
'----------------------------------------------------------------
'
Public Property Let BackColor(ByVal Value As OLE_COLOR)
   UserControl.BackColor = Value
   PropertyChanged "BackColor"
   Refresh
End Property

'----------------------------------------------------------------
' Procedure : BorderStyle
' Purpose   : Returns the border style
'----------------------------------------------------------------
'
Public Property Get BorderStyle() As BorderStyle
   BorderStyle = m_lBorderStyle
End Property

'----------------------------------------------------------------
' Procedure : BorderStyle
' Purpose   : Sets the border style
'----------------------------------------------------------------
'
Public Property Let BorderStyle(ByVal Value As BorderStyle)
   m_lBorderStyle = Value
   SetBorder
   PropertyChanged "BorderStyle"
End Property

'----------------------------------------------------------------
' Procedure : Clear
' Purpose   : Removes all items
'----------------------------------------------------------------
'
Public Sub Clear()

   SendMessage m_lListBox, LB_RESETCONTENT, 0, ByVal 0&
   Set m_oItems = New Collection

End Sub

'----------------------------------------------------------------
' Procedure : Count
' Purpose   : Returns the number of items in the ListBox
'----------------------------------------------------------------
'
Public Property Get Count() As Long
Attribute Count.VB_MemberFlags = "400"
   Count = m_oItems.Count
End Property

'----------------------------------------------------------------
' Procedure : Font
' Purpose   : Returns the control font
'----------------------------------------------------------------
'
Public Property Get Font() As StdFont
Attribute Font.VB_UserMemId = -512
   Set Font = UserControl.Font
End Property

'----------------------------------------------------------------
' Procedure : Font
' Purpose   : Sets the control font
'----------------------------------------------------------------
'
Public Property Set Font(ByVal Value As StdFont)
   Set UserControl.Font = Value
   PropertyChanged "Font"

   If Ambient.UserMode Then
      Dim oItem As ListItem
      
      For Each oItem In m_oItems
         ICallback_RecalculateItemHeight ObjPtr(oItem)
      Next
      
   End If

End Property

'----------------------------------------------------------------
' Procedure : ForeColor
' Purpose   : Returns the default text color
'----------------------------------------------------------------
'
Public Property Get ForeColor() As OLE_COLOR
Attribute ForeColor.VB_UserMemId = -513
   ForeColor = UserControl.ForeColor
End Property

'----------------------------------------------------------------
' Procedure : ForeColor
' Purpose   : Sets the default text color
'----------------------------------------------------------------
'
Public Property Let ForeColor(ByVal Value As OLE_COLOR)
   UserControl.ForeColor = Value
   PropertyChanged "ForeColor"
End Property

'----------------------------------------------------------------
' Procedure : hWnd
' Purpose   : Returns the ListBox window handle
'----------------------------------------------------------------
'
Public Property Get hWnd() As Long
Attribute hWnd.VB_UserMemId = -515
    hWnd = m_lListBox
End Property

'----------------------------------------------------------------
' Procedure : Item
' Purpose   : Returns an item
'----------------------------------------------------------------
'
Public Property Get Item(ByVal Index As Variant) As ListItem
Attribute Item.VB_UserMemId = 0
Attribute Item.VB_MemberFlags = "600"
    Set Item = m_oItems(Index)
End Property

'----------------------------------------------------------------
' Procedure : ItemRenderer
' Purpose   : Returns the current item renderer
'----------------------------------------------------------------
'
Public Property Get ItemRenderer() As IDrawListItem
   Set ItemRenderer = m_oDraw
End Property

'----------------------------------------------------------------
' Procedure : DrawItemCallback
' Purpose   : Sets the item renderer
'----------------------------------------------------------------
'
Public Property Set ItemRenderer(ByVal Value As IDrawListItem)
   Set m_oDraw = Value
End Property

'----------------------------------------------------------------
' Procedure : NewEnum
' Purpose   : Returns an enumerator object to be used with For..Each
'----------------------------------------------------------------
'
Public Function NewEnum() As IUnknown
Attribute NewEnum.VB_UserMemId = -4
Attribute NewEnum.VB_MemberFlags = "40"
   Set NewEnum = m_oItems.[_NewEnum]
End Function

'----------------------------------------------------------------
' Procedure : Refresh
' Purpose   : Repaints the control
'----------------------------------------------------------------
'
Public Sub Refresh()
Attribute Refresh.VB_UserMemId = -550
   RedrawWindow m_lListBox, ByVal 0&, 0&, RDW_INVALIDATE Or RDW_UPDATENOW Or RDW_ERASENOW Or RDW_ERASE
End Sub

'----------------------------------------------------------------
' Procedure : Remove
' Purpose   : Removes an item
'----------------------------------------------------------------
'
Public Sub Remove(ByVal Index As Variant)
Dim oItem As ListItem
Dim lIdx As Long

   Set oItem = m_oItems(Index)
   lIdx = SendMessage(m_lListBox, LB_FINDSTRINGEXACT, -1, ByVal ObjPtr(oItem))
   
   SendMessage m_lListBox, LB_DELETESTRING, lIdx, ByVal 0&
   m_oItems.Remove Index
   
End Sub

'----------------------------------------------------------------
' Procedure : SelBackColor
' Purpose   : Returns the default background color of selected items
'----------------------------------------------------------------
'
Public Property Get SelBackColor() As OLE_COLOR
   SelBackColor = m_lSelBack
End Property

'----------------------------------------------------------------
' Procedure : SelBackColor
' Purpose   : Sets the default background color of selected items
'----------------------------------------------------------------
'
Public Property Let SelBackColor(ByVal Value As OLE_COLOR)
   m_lSelBack = Value
   PropertyChanged "SelBackColor"
   Refresh
End Property

'----------------------------------------------------------------
' Procedure : SelectedIndex
' Purpose   : Returns the index of the selected item
'----------------------------------------------------------------
'
Public Property Get SelectedIndex() As Long
Attribute SelectedIndex.VB_MemberFlags = "400"
   SelectedIndex = SendMessage(m_lListBox, LB_GETCURSEL, 0, ByVal 0&) + 1
End Property

'----------------------------------------------------------------
' Procedure : SelectedIndex
' Purpose   : Sets the selected item
'----------------------------------------------------------------
'
Public Property Let SelectedIndex(ByVal Value As Long)
   SendMessage m_lListBox, LB_SETCURSEL, Value - 1, ByVal 0&
End Property

'----------------------------------------------------------------
' Procedure : SelectedItem
' Purpose   : Returns the selected item
'----------------------------------------------------------------
'
Public Property Get SelectedItem() As ListItem
Attribute SelectedItem.VB_MemberFlags = "400"
   If SelectedIndex > 0 Then
      Set SelectedItem = m_oItems(SelectedIndex)
   End If
End Property

'----------------------------------------------------------------
' Procedure : SelectedItem
' Purpose   : Sets the selected item
'----------------------------------------------------------------
'
Public Property Set SelectedItem(ByVal Value As ListItem)
   SelectedIndex = ICallback_IndexFromItem(ObjPtr(Value)) + 1
End Property

'----------------------------------------------------------------
' Procedure : SelForeColor
' Purpose   : Returns the default text color of selected items
'----------------------------------------------------------------
'
Public Property Get SelForeColor() As OLE_COLOR
   SelForeColor = m_lSelFore
End Property

'----------------------------------------------------------------
' Procedure : SelForeColor
' Purpose   : Sets the default text color of selected items
'----------------------------------------------------------------
'
Public Property Let SelForeColor(ByVal Value As OLE_COLOR)
   m_lSelFore = Value
   PropertyChanged "SelForeColor"
End Property

'----------------------------------------------------------------
' Procedure : Sorted
' Purpose   : Returns wheter the items are sorted
'----------------------------------------------------------------
'
Public Property Get Sorted() As Boolean
   Sorted = m_bSorted
End Property

'----------------------------------------------------------------
' Procedure : Sorted
' Purpose   : Sets whether items are sorted
'----------------------------------------------------------------
'
Public Property Let Sorted(ByVal Value As Boolean)
   If Ambient.UserMode Then Err.Raise 382
   m_bSorted = Value
   PropertyChanged "Sorted"
End Property

Private Sub UserControl_AmbientChanged(PropertyName As String)

   If PropertyName = "DisplayName" Then
      Remove 1
      Add Ambient.DisplayName
   End If
   
End Sub

Private Sub UserControl_KeyDown(KeyCode As Integer, Shift As Integer)

   RaiseEvent KeyDown(KeyCode, Shift)
    
   If KeyCode Then
      CallWindowProc m_lWndProcLB, m_lListBox, WM_KEYDOWN, KeyCode, 0
   End If
   
End Sub

Private Sub UserControl_KeyPress(KeyAscii As Integer)

   RaiseEvent KeyPress(KeyAscii)
   
   If KeyAscii Then
      CallWindowProc m_lWndProcLB, m_lListBox, WM_CHAR, KeyAscii, 0
   End If
   
End Sub

Private Sub UserControl_KeyUp(KeyCode As Integer, Shift As Integer)

   RaiseEvent KeyUp(KeyCode, Shift)
   
   If KeyCode Then
      Call CallWindowProc(m_lWndProcLB, m_lListBox, WM_KEYUP, KeyCode, 0)
   End If
   
End Sub

Private Sub UserControl_Initialize()

   Set m_oItems = New Collection
   Set m_oDraw = New DefaultItemDrawer
   
   m_lWndProc = SubclassWindow(UserControl.hWnd, Me)
    
   Set m_oCanvas = New Canvas
   Set m_oCanvas.PictureBox = picCanvas
    
End Sub

Private Sub UserControl_InitProperties()
   
   Set UserControl.Font = Ambient.Font
   UserControl.ForeColor = vbWindowText
   UserControl.BackColor = vbWindowBackground
   m_lSelFore = vbHighlightText
   m_lSelBack = vbHighlight
   m_lBorderStyle = bs3D
   
   CreateListBox
   
   Add Ambient.DisplayName
   
End Sub

Private Sub UserControl_MouseDown(Button As Integer, Shift As Integer, X As Single, Y As Single)
    RaiseEvent MouseDown(Button, Shift, X, Y)
End Sub

Private Sub UserControl_MouseMove(Button As Integer, Shift As Integer, X As Single, Y As Single)
    RaiseEvent MouseMove(Button, Shift, X, Y)
End Sub

Private Sub UserControl_MouseUp(Button As Integer, Shift As Integer, X As Single, Y As Single)
    RaiseEvent MouseUp(Button, Shift, X, Y)
End Sub

Private Sub UserControl_ReadProperties(PropBag As PropertyBag)
    
   Set UserControl.Font = PropBag.ReadProperty("Font", Ambient.Font)
   UserControl.ForeColor = PropBag.ReadProperty("ForeColor", Ambient.ForeColor)
   UserControl.BackColor = PropBag.ReadProperty("BackColor", Ambient.BackColor)
   m_lSelFore = PropBag.ReadProperty("SelForeColor", vbHighlightText)
   m_lSelBack = PropBag.ReadProperty("SelBackColor", vbHighlight)
   m_bSorted = PropBag.ReadProperty("Sorted", False)
   m_lBorderStyle = PropBag.ReadProperty("BorderStyle", bs3D)
   
   SetBorder
   CreateListBox
   
   If Not Ambient.UserMode Then
      Add Ambient.DisplayName
   End If
   
End Sub

Private Sub UserControl_Resize()

   SetWindowPos m_lListBox, 0, 0, 0, ScaleWidth, ScaleHeight, SWP_NOMOVE Or SWP_NOZORDER
   RedrawWindow m_lListBox, ByVal 0&, 0&, RDW_INVALIDATE Or RDW_NOERASE
   
End Sub

Private Sub UserControl_Terminate()
   
   ' Destroy the listbox control
   DestroyListBox
   
   ' Unsubclass the control
   UnsubclassWindow Me.hWnd
   
End Sub

Private Sub UserControl_WriteProperties(PropBag As PropertyBag)
   
   PropBag.WriteProperty "Font", UserControl.Font, Ambient.Font
   PropBag.WriteProperty "ForeColor", UserControl.ForeColor, Ambient.ForeColor
   PropBag.WriteProperty "BackColor", UserControl.BackColor, Ambient.BackColor
   PropBag.WriteProperty "SelForeColor", m_lSelFore, vbHighlightText
   PropBag.WriteProperty "SelBackColor", m_lSelBack, vbHighlight
   PropBag.WriteProperty "Sorted", m_bSorted, False
   PropBag.WriteProperty "BorderStyle", m_lBorderStyle, bs3D

End Sub

'----------------------------------------------------------------
' Procedure : OnCompareItems
' Purpose   : Handles WM_COMPAREITEM message
'----------------------------------------------------------------
'
Private Function onCompareItems(ByVal lParam As Long) As Long
Dim tCI As COMPAREITEMSTRUCT
Dim oItem1 As ListItem
Dim oItem2 As ListItem

   ' Get the COMPAREITEMSTRUCT from the pointer
   MoveMemory tCI, ByVal lParam, Len(tCI)
   
   ' Get the items
   Set oItem1 = ItemFromPtr(tCI.itemData1)
   Set oItem2 = ItemFromPtr(tCI.itemData2)
   MoveMemory oItem2, tCI.itemData2, 4&
   
   ' Compare the items
   onCompareItems = StrComp(oItem1.Text, oItem2.Text, vbTextCompare)
   
End Function

'----------------------------------------------------------------
' Procedure : CreateListBox
' Purpose   : Creates the ListBox window
'----------------------------------------------------------------
'
Private Function CreateListBox()
Dim lStyle As Long
Dim lExStyle As Long

   lStyle = LBS_NOINTEGRALHEIGHT Or _
            LBS_OWNERDRAWVARIABLE Or LBS_NOTIFY Or _
            WS_VISIBLE Or WS_CHILD Or WS_GROUP Or _
            WS_TABSTOP Or WS_VSCROLL
           
   If m_bSorted Then
      lStyle = lStyle Or LBS_SORT
   End If
   
   If Ambient.RightToLeft Then
      lExStyle = lExStyle Or WS_EX_LEFTSCROLLBAR Or WS_EX_RTLREADING
   End If
   
   ' Create the ListBox window
   m_lListBox = CreateWindowEx(lExStyle, "ListBox", _
                vbNullString, lStyle, 0, 0, _
                ScaleWidth, ScaleHeight, _
                UserControl.hWnd, 1, App.hInstance, ByVal 0&)
            
   If m_lListBox = 0 Then
      Err.Raise vbObjectError, , "Cannot create ListBox control"
   Else
   
      If Ambient.UserMode Then
         ' Subclass the ListBox window
         m_lWndProcLB = SubclassWindow(m_lListBox, Me)
      End If
       
   End If
    
End Function

'----------------------------------------------------------------
' Procedure : DestroyListBox
' Purpose   : Destroys the ListBox window
'----------------------------------------------------------------
'
Private Function DestroyListBox()

   ' Unsubclass the listbox
   UnsubclassWindow m_lListBox
  
   ' Destroy the listbox
   DestroyWindow m_lListBox
   m_lListBox = 0
    
End Function

'----------------------------------------------------------------
' Procedure : OnDrawItem
' Purpose   : Handles WM_DRAWITEM message
'----------------------------------------------------------------
'
Private Sub onDrawItem(ByVal lParam As Long)
Dim tDI As DRAWITEMSTRUCT
Dim oItem As ListItem

   ' Get the DRAWITEMSTRUCT from the pointer
   MoveMemory tDI, ByVal lParam, Len(tDI)
   
   If tDI.ItemData Then
   
      ' Get the item
      Set oItem = ItemFromPtr(tDI.ItemData)
      
      ' Initialize the canvas
      InitCanvas oItem, _
         tDI.rcItem.Right - tDI.rcItem.Left, _
         tDI.rcItem.Bottom - tDI.rcItem.Top
   
      ' Draw the item using the current renderer
      m_oDraw.DrawItem oItem, _
            tDI.itemState And ODS_SELECTED, _
            tDI.itemState And ODS_FOCUS, _
            m_oCanvas
      
      BitBlt tDI.hDC, tDI.rcItem.Left, tDI.rcItem.Top, _
         tDI.rcItem.Right - tDI.rcItem.Left, _
         tDI.rcItem.Bottom - tDI.rcItem.Top, _
         picCanvas.hDC, 0, 0, vbSrcCopy
   
   End If
   
End Sub

'----------------------------------------------------------------
' Procedure : InitCanvas
' Purpose   : Initializes the PictureBox used to render the items
'----------------------------------------------------------------
'
Private Sub InitCanvas(Item As ListItem, W As Long, H As Long)
Dim oFont As IFont
Dim oNewFont As IFont

   With picCanvas
        
      ' Clone the font
      Set oFont = UserControl.Font
      oFont.Clone oNewFont
      
      ' Clear the picture box
      .Cls
      
      ' Set the properties
      Set .Font = oNewFont
      .ForeColor = Item.ForeColor
      .BackColor = Item.BackColor
      .ScaleMode = vbPixels
      .Width = W
      .Height = H
   End With

End Sub

'----------------------------------------------------------------
' Procedure : OnMeasureItem
' Purpose   : Handles the WM_MEASUREITEM message
'----------------------------------------------------------------
'
Private Sub onMeasureItem(ByVal lParam As Long)
Dim tMI As MEASUREITEMSTRUCT
Dim oItem As ListItem

   MoveMemory tMI, ByVal lParam, Len(tMI)
   
   If tMI.ItemData Then
   
      Set oItem = ItemFromPtr(tMI.ItemData)
      
      InitCanvas oItem, ScaleWidth, ScaleHeight
      m_oDraw.MeasureItem oItem, tMI.ItemHeight, m_oCanvas
      
      MoveMemory ByVal lParam, tMI, Len(tMI)
   
   End If
   
End Sub

'----------------------------------------------------------------
' Procedure : SetBorder
' Purpose   : Sets the control border
'----------------------------------------------------------------
'
Private Sub SetBorder()
Dim lStyle As Long
Dim lExStyle As Long

   lStyle = GetWindowLong(UserControl.hWnd, GWL_STYLE)
   lExStyle = GetWindowLong(UserControl.hWnd, GWL_EXSTYLE)

   Select Case m_lBorderStyle
      Case bsNone
         lStyle = lStyle And (Not WS_BORDER)
         lExStyle = lExStyle And Not (WS_EX_CLIENTEDGE Or WS_EX_STATICEDGE)
      Case bsSingle
         lStyle = lStyle Or WS_BORDER
         lExStyle = lExStyle And Not (WS_EX_CLIENTEDGE Or WS_EX_STATICEDGE)
      Case bsFlat
         lStyle = lStyle And (Not WS_BORDER)
         lExStyle = lExStyle And (Not WS_EX_CLIENTEDGE)
         lExStyle = lExStyle Or WS_EX_STATICEDGE
      Case bs3D
         lStyle = lStyle And (Not WS_BORDER)
         lExStyle = lExStyle And (Not WS_EX_STATICEDGE)
         lExStyle = lExStyle Or WS_EX_CLIENTEDGE
   End Select
   
   SetWindowLong UserControl.hWnd, GWL_STYLE, lStyle
   SetWindowLong UserControl.hWnd, GWL_EXSTYLE, lExStyle
   
   SetWindowPos UserControl.hWnd, 0, 0, 0, 0, 0, SWP_NOMOVE Or SWP_NOSIZE Or SWP_NOZORDER Or SWP_FRAMECHANGED
   
End Sub

'----------------------------------------------------------------
' Procedure : EnsureVisible
' Purpose   : Makes an item visible
'----------------------------------------------------------------
'
Private Sub ICallback_EnsureVisible(ByVal ItemPtr As Long)
Dim lIdx As Long
   
   lIdx = ICallback_IndexFromItem(ItemPtr)
   SendMessage m_lListBox, LB_SETTOPINDEX, lIdx, ByVal 0&
   
End Sub

'----------------------------------------------------------------
' Procedure : IndexFromItem
' Purpose   : Returns the index of a ListItem
'----------------------------------------------------------------
'
Private Function ICallback_IndexFromItem(ByVal ItemPtr As Long) As Long
   ICallback_IndexFromItem = SendMessage(m_lListBox, LB_FINDSTRINGEXACT, -1, ByVal ItemPtr)
End Function

'----------------------------------------------------------------
' Procedure : RecalculateItemHeight
' Purpose   : Recalculates the item height
'----------------------------------------------------------------
'
Private Sub ICallback_RecalculateItemHeight(ByVal ItemPtr As Long)
Dim lHeight As Long
Dim oItem As ListItem
Dim lIdx As Long

   lIdx = ICallback_IndexFromItem(ItemPtr)
   
   If lIdx >= 0 Then

      Set oItem = ItemFromPtr(ItemPtr)
      
      InitCanvas oItem, ScaleWidth, ScaleHeight
      m_oDraw.MeasureItem oItem, lHeight, m_oCanvas
      
      SendMessage m_lListBox, LB_SETITEMHEIGHT, lIdx, ByVal lHeight
      
      Refresh
      
   End If
   
End Sub

'----------------------------------------------------------------
' Procedure : ICallback_Refresh
' Purpose   : Redraws the control
'----------------------------------------------------------------
'
Private Sub ICallback_Refresh()
   Refresh
End Sub

Private Function ICallback_WindowProc(ByVal Wnd As Long, ByVal uMsg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long
Static Focusing As Boolean
Dim iButton As Integer, X As Single, Y As Single, iShift As Integer

   If Wnd = m_lListBox Then
   
      Select Case uMsg
            
         Case WM_SETFOCUS
            
            Focusing = True
            
            ' This code sets the focus to the UserControl
            ' when the ListBox gets the focus
            
            SendMessage UserControl.hWnd, &H100E, 35, 0&
            SendMessage UserControl.hWnd, WM_SETFOCUS, wParam, 0&
            SendMessage UserControl.hWnd, &H100E, 14, 0&
            SendMessage UserControl.hWnd, WM_KILLFOCUS, Wnd, 0&
            SendMessage UserControl.hWnd, &H100E, 18, 0&
            SendMessage UserControl.hWnd, &H100E, 36, 0&
            
            Focusing = False
         
            Case WM_LBUTTONDOWN, WM_MBUTTONDOWN, WM_RBUTTONDOWN
                iButton = (Abs(uMsg = WM_LBUTTONDOWN)) * vbLeftButton + (Abs(uMsg = WM_RBUTTONDOWN)) * vbRightButton + (Abs(uMsg = WM_MBUTTONDOWN)) * vbMiddleButton
                iShift = 0 'wParam
                If (lParam And &H8000&) = &H8000& Then
                    X = -(&H8000& - (lParam And &H7FFF&))
                Else
                    X = (lParam And &HFFFF&)
                End If
                If (lParam And &H80000000) = &H80000000 Then
                    Y = -(&H8000& - (lParam And &H7FFF0000) \ &H10000)
                Else
                    Y = (lParam \ &H10000)
                End If
                RaiseEvent MouseDown(iButton, iShift, X, Y)
                             
            Case WM_MOUSEMOVE
                iButton = Abs(GetAsyncKeyState(vbKeyLButton) <> 0) * vbLeftButton + Abs(GetAsyncKeyState(vbKeyRButton) <> 0) * vbRightButton + Abs(GetAsyncKeyState(vbKeyMButton) <> 0) * vbMiddleButton
                iShift = 0 'wParam
                If (lParam And &H8000&) = &H8000& Then
                    X = -(&H8000& - (lParam And &H7FFF&))
                Else
                    X = (lParam And &HFFFF&)
                End If
                If (lParam And &H80000000) = &H80000000 Then
                    Y = -(&H8000& - (lParam And &H7FFF0000) \ &H10000)
                Else
                    Y = (lParam \ &H10000)
                End If
                RaiseEvent MouseMove(iButton, iShift, X, Y)
             
            Case WM_LBUTTONUP, WM_MBUTTONUP, WM_RBUTTONUP
                iButton = (Abs(uMsg = WM_LBUTTONDOWN)) * vbLeftButton + (Abs(uMsg = WM_RBUTTONDOWN)) * vbRightButton + (Abs(uMsg = WM_MBUTTONDOWN)) * vbMiddleButton
                iShift = 0 'wParam
                If (lParam And &H8000&) = &H8000& Then
                    X = -(&H8000& - (lParam And &H7FFF&))
                Else
                    X = (lParam And &HFFFF&)
                End If
                If (lParam And &H80000000) = &H80000000 Then
                    Y = -(&H8000& - (lParam And &H7FFF0000) \ &H10000)
                Else
                    Y = (lParam \ &H10000)
                End If
                RaiseEvent MouseUp(iButton, iShift, X, Y)
      End Select
      
      ' Call the previous window procedure
      ICallback_WindowProc = CallWindowProc(m_lWndProcLB, Wnd, uMsg, wParam, lParam)
   
      Select Case uMsg
         Case WM_VSCROLL
            RaiseEvent Scroll
      End Select
      
   Else

      Select Case uMsg
      
         Case WM_COMPAREITEM
         
            ICallback_WindowProc = onCompareItems(lParam)
            
            Exit Function
            
         Case WM_CTLCOLORLISTBOX
            Static hBr As Long
            
            DeleteObject hBr
            hBr = CreateSolidBrush(OleClr2ClrRef(UserControl.BackColor))
            
            ICallback_WindowProc = hBr
            
            Exit Function
            
         Case WM_DRAWITEM
         
            onDrawItem lParam
            
            Exit Function
            
         Case WM_MEASUREITEM
         
            onMeasureItem lParam
            
            Exit Function
         
         Case WM_COMMAND
      
            Select Case wParam \ &H10000
               Case LBN_DBLCLK
                  RaiseEvent DblClick
               Case LBN_SELCHANGE
                  RaiseEvent Click
            End Select
      
            Exit Function
         
         Case WM_SETFOCUS
            
            If Not Focusing Then
               
               Focusing = True
               
               ' Set the focus to the listbox
               SetFocusAPI m_lListBox
               
               Focusing = False
               
            End If
            
         Case WM_KILLFOCUS
         
         ' Send the message to the ListBox
         SendMessage m_lListBox, WM_KILLFOCUS, wParam, 0
         
      End Select
   
      ICallback_WindowProc = CallWindowProc(m_lWndProc, Wnd, uMsg, wParam, lParam)
      
   End If
   
End Function

'----------------------------------------------------------------
' Procedure : ICallback_SelectItem
' Purpose   : Sets the selected item
'----------------------------------------------------------------
'
Private Sub ICallback_SelectItem(ByVal ItemPtr As Long)
   SelectedIndex = ICallback_IndexFromItem(ItemPtr)
End Sub

